@@ -121,4 +121,19 @@ typedef enum
   GSD_NUM_LOCK_STATE_OFF
 } GsdNumLockState;
 
+typedef enum
+{
+  GSD_INPUT_SOURCES_SWITCHER_OFF,
+  GSD_INPUT_SOURCES_SWITCHER_SHIFT_L,
+  GSD_INPUT_SOURCES_SWITCHER_ALT_L,
+  GSD_INPUT_SOURCES_SWITCHER_CTRL_L,
+  GSD_INPUT_SOURCES_SWITCHER_SHIFT_R,
+  GSD_INPUT_SOURCES_SWITCHER_ALT_R,
+  GSD_INPUT_SOURCES_SWITCHER_CTRL_R,
+  GSD_INPUT_SOURCES_SWITCHER_ALT_SHIFT_L,
+  GSD_INPUT_SOURCES_SWITCHER_ALT_SHIFT_R,
+  GSD_INPUT_SOURCES_SWITCHER_CTRL_SHIFT_L,
+  GSD_INPUT_SOURCES_SWITCHER_CTRL_SHIFT_R
+} GsdInputSourcesSwitcher;
+
 #endif /* __gsd_enums_h__ */
