@@ -76,6 +76,7 @@ static void
 grab_key_real (guint      keycode,
                GdkWindow *root,
                gboolean   grab,
+               gboolean   synchronous,
                XIGrabModifiers *mods,
                int        num_mods)
 {
@@ -96,7 +97,7 @@ grab_key_real (guint      keycode,
                                keycode,
                                GDK_WINDOW_XID (root),
                                GrabModeAsync,
-                               GrabModeAsync,
+                               synchronous ? GrabModeSync : GrabModeAsync,
                                False,
                                &evmask,
                                num_mods,
@@ -132,10 +133,11 @@ grab_key_real (guint      keycode,
  * operations with one flush only.
  */
 #define N_BITS 32
-void
-grab_key_unsafe (Key                 *key,
-                 gboolean             grab,
-                 GSList              *screens)
+static void
+grab_key_internal (Key             *key,
+                   gboolean         grab,
+                   GsdKeygrabFlags  flags,
+                   GSList          *screens)
 {
         int     indexes[N_BITS]; /* indexes of bits we need to flip */
         int     i;
@@ -161,7 +163,8 @@ grab_key_unsafe (Key                 *key,
          * The exception is the XFree86 keys and the Function keys
          * (which are useful to grab without a modifier).
          */
-        if ((modifiers & gsd_used_mods) == 0 &&
+        if (!(flags & GSD_KEYGRAB_ALLOW_UNMODIFIED) &&
+            (modifiers & gsd_used_mods) == 0 &&
             !IN_RANGE(key->keysym, XF86KEYS_RANGE_MIN, XF86KEYS_RANGE_MAX) &&
             !IN_RANGE(key->keysym, FKEYS_RANGE_MIN, FKEYS_RANGE_MAX) &&
              key->keysym != GDK_KEY_Pause &&
@@ -224,6 +227,7 @@ grab_key_unsafe (Key                 *key,
                         grab_key_real (*code,
                                        gdk_screen_get_root_window (screen),
                                        grab,
+                                       flags & GSD_KEYGRAB_SYNCHRONOUS,
                                        (XIGrabModifiers *) all_mods->data,
                                        all_mods->len);
                 }
@@ -231,6 +235,21 @@ grab_key_unsafe (Key                 *key,
         g_array_free (all_mods, TRUE);
 }
 
+void
+grab_key_unsafe (Key             *key,
+                 GsdKeygrabFlags  flags,
+                 GSList          *screens)
+{
+        grab_key_internal (key, TRUE, flags, screens);
+}
+
+void
+ungrab_key_unsafe (Key    *key,
+                   GSList *screens)
+{
+        grab_key_internal (key, FALSE, 0, screens);
+}
+
 static gboolean
 have_xkb (Display *dpy)
 {
