@@ -71,6 +71,8 @@
 #define KEY_BELL_DURATION  "bell-duration"
 #define KEY_BELL_MODE      "bell-mode"
 
+#define KEY_SWITCHER "input-sources-switcher"
+
 #define GNOME_DESKTOP_INTERFACE_DIR "org.gnome.desktop.interface"
 
 #define KEY_GTK_IM_MODULE    "gtk-im-module"
@@ -107,6 +109,9 @@ struct GsdKeyboardManagerPrivate
         GdkDeviceManager *device_manager;
         guint device_added_id;
         guint device_removed_id;
+
+        gboolean input_sources_switcher_spawned;
+        GPid input_sources_switcher_pid;
 };
 
 static void     gsd_keyboard_manager_class_init  (GsdKeyboardManagerClass *klass);
@@ -1002,6 +1007,47 @@ apply_all_settings (GsdKeyboardManager *manager)
 	apply_numlock (manager);
 }
 
+static void
+set_input_sources_switcher (GsdKeyboardManager *manager,
+                            gboolean            state)
+{
+        if (state) {
+                GError *error = NULL;
+                char *args[2];
+
+                if (manager->priv->input_sources_switcher_spawned)
+                        set_input_sources_switcher (manager, FALSE);
+
+                args[0] = LIBEXECDIR "/gsd-input-sources-switcher";
+                args[1] = NULL;
+
+                g_spawn_async (NULL, args, NULL,
+                               0, NULL, NULL,
+                               &manager->priv->input_sources_switcher_pid, &error);
+
+                manager->priv->input_sources_switcher_spawned = (error == NULL);
+
+                if (error) {
+                        g_warning ("Couldn't spawn %s: %s", args[0], error->message);
+                        g_error_free (error);
+                }
+        } else if (manager->priv->input_sources_switcher_spawned) {
+                kill (manager->priv->input_sources_switcher_pid, SIGHUP);
+                g_spawn_close_pid (manager->priv->input_sources_switcher_pid);
+                manager->priv->input_sources_switcher_spawned = FALSE;
+        }
+}
+
+static gboolean
+enable_switcher (GsdKeyboardManager *manager)
+{
+        GsdInputSourcesSwitcher switcher;
+
+        switcher = g_settings_get_enum (manager->priv->settings, KEY_SWITCHER);
+
+        return switcher != GSD_INPUT_SOURCES_SWITCHER_OFF;
+}
+
 static void
 settings_changed (GSettings          *settings,
                   const char         *key,
@@ -1023,6 +1069,8 @@ settings_changed (GSettings          *settings,
 		 g_strcmp0 (key, KEY_DELAY) == 0) {
 		g_debug ("Key repeat setting '%s' changed, applying key repeat settings", key);
 		apply_repeat (manager);
+        } else if (g_strcmp0 (key, KEY_SWITCHER) == 0) {
+                set_input_sources_switcher (manager, enable_switcher (manager));
 	} else {
 		g_warning ("Unhandled settings change, key '%s'", key);
 	}
@@ -1072,6 +1120,67 @@ set_devicepresence_handler (GsdKeyboardManager *manager)
         manager->priv->device_manager = device_manager;
 }
 
+static void
+create_sources_from_current_xkb_config (GSettings *settings)
+{
+        GVariantBuilder builder;
+        XkbRF_VarDefsRec *xkb_var_defs;
+        gchar *tmp;
+        gchar **layouts = NULL;
+        gchar **variants = NULL;
+        guint i, n;
+
+        gnome_xkb_info_get_var_defs (&tmp, &xkb_var_defs);
+        g_free (tmp);
+
+        if (xkb_var_defs->layout)
+                layouts = g_strsplit (xkb_var_defs->layout, ",", 0);
+        if (xkb_var_defs->variant)
+                variants = g_strsplit (xkb_var_defs->variant, ",", 0);
+
+        gnome_xkb_info_free_var_defs (xkb_var_defs);
+
+        if (!layouts)
+                goto out;
+
+        if (variants && variants[0])
+                n = MIN (g_strv_length (layouts), g_strv_length (variants));
+        else
+                n = g_strv_length (layouts);
+
+        g_variant_builder_init (&builder, G_VARIANT_TYPE ("a(ss)"));
+        for (i = 0; i < n && layouts[i][0]; ++i) {
+                if (variants && variants[i] && variants[i][0])
+                        tmp = g_strdup_printf ("%s+%s", layouts[i], variants[i]);
+                else
+                        tmp = g_strdup (layouts[i]);
+
+                g_variant_builder_add (&builder, "(ss)", INPUT_SOURCE_TYPE_XKB, tmp);
+                g_free (tmp);
+        }
+        g_settings_set_value (settings, KEY_INPUT_SOURCES, g_variant_builder_end (&builder));
+out:
+        g_strfreev (layouts);
+        g_strfreev (variants);
+}
+
+static void
+maybe_create_input_sources (GsdKeyboardManager *manager)
+{
+        GVariant *sources;
+        gboolean gdm;
+        gsize n;
+
+        gdm = g_getenv ("RUNNING_UNDER_GDM") != NULL;
+
+        sources = g_settings_get_value (manager->priv->input_sources_settings, KEY_INPUT_SOURCES);
+        n = g_variant_n_children (sources);
+        g_variant_unref (sources);
+
+        if (n < 1 || gdm)
+                create_sources_from_current_xkb_config (manager->priv->input_sources_settings);
+}
+
 static gboolean
 start_keyboard_idle_cb (GsdKeyboardManager *manager)
 {
@@ -1089,6 +1198,8 @@ start_keyboard_idle_cb (GsdKeyboardManager *manager)
         manager->priv->interface_settings = g_settings_new (GNOME_DESKTOP_INTERFACE_DIR);
         manager->priv->xkb_info = gnome_xkb_info_new ();
 
+        maybe_create_input_sources (manager);
+
 #ifdef HAVE_IBUS
         /* We don't want to touch IBus until we are sure this isn't a
            fallback session. */
@@ -1111,6 +1222,7 @@ start_keyboard_idle_cb (GsdKeyboardManager *manager)
                           G_CALLBACK (apply_input_sources_settings), manager);
 
 	install_xkb_filter (manager);
+        set_input_sources_switcher (manager, enable_switcher (manager));
 
         gnome_settings_profile_end (NULL);
 
@@ -1160,6 +1272,8 @@ gsd_keyboard_manager_stop (GsdKeyboardManager *manager)
         }
 
 	remove_xkb_filter (manager);
+
+        set_input_sources_switcher (manager, FALSE);
 }
 
 static void
